
import * as React from 'react'
import ReactDOM from 'react-dom/client'
import App from './App'
import './index.css'
import { ErrorBoundary } from './components/ErrorBoundary'
import { initThemeHandler } from './components/theme/IframeThemeHandler'
import { Toaster } from 'sonner'
import { supabase } from '@/integrations/supabase/client'
import { DataProvider } from './providers/DataProvider'
import { initMemoryUtils, cleanupMemoryUtils } from './utils/memoryUtils'
import { storageManager } from './utils/cacheUtils'

// Add version info for cache busting
const appVersion = Date.now().toString();
console.log(`App version: ${appVersion}`);

// Fix: Ensure we have a valid DOM element before creating root
const rootElement = document.getElementById('root')

if (!rootElement) {
  throw new Error('Root element not found')
}

// Apply theme class from localStorage if available
const applyStoredTheme = () => {
  try {
    const theme = storageManager.getItem<string>('funnel-depot-theme');
    if (theme === 'dark' || (theme === 'system' && window.matchMedia('(prefers-color-scheme: dark)').matches)) {
      document.documentElement.classList.add('dark')
    } else {
      document.documentElement.classList.add('light')
    }
  } catch (e) {
    console.error('Failed to apply stored theme:', e)
    document.documentElement.classList.add('light') // Default to light theme if error
  }
}

// Initialize storage buckets for social sharing images with unlimited quota
const initSocialSharingStorage = async () => {
  try {
    // Check if the public bucket exists, and create it if it doesn't
    const { data: buckets } = await supabase.storage.listBuckets();
    const publicBucket = buckets?.find(bucket => bucket.name === 'public');
    
    if (!publicBucket) {
      // Create the public bucket for social sharing images with unlimited quota
      await supabase.storage.createBucket('public', {
        public: true, // Make it publicly accessible
        fileSizeLimit: 50000000, // 50MB instead of 5MB
      });
      console.log('Created public storage bucket for social sharing images');
    } else {
      // Update existing bucket to ensure unlimited quota
      await supabase.storage.updateBucket('public', {
        public: true,
        fileSizeLimit: 50000000, // 50MB for much larger file support
      });
      console.log('Updated public storage bucket with increased quota');
    }
  } catch (err) {
    console.error('Error initializing storage for social sharing:', err);
  }
};

// Apply theme before initial render to prevent flash
applyStoredTheme()

// Remove initial loader once React is mounted
const removeInitialLoader = () => {
  const loader = document.querySelector('.initial-loader');
  if (loader) {
    loader.classList.add('fade-out');
    setTimeout(() => {
      loader.remove();
    }, 300);
  }
};

// Initialize the theme handler for iframes - but with a delay to ensure the app is loaded first
const initThemeWithDelay = () => {
  try {
    initThemeHandler();
    initSocialSharingStorage();
    
    // Initialize memory utilities in development
    if (process.env.NODE_ENV === 'development') {
      initMemoryUtils();
    }
    
    // Clean up expired items in localStorage
    storageManager.cleanup();
  } catch (e) {
    console.error('Failed to initialize app services:', e);
  }
};

// Helper function to check if toasts should be visible on current route
const shouldShowToastsOnCurrentRoute = () => {
  // Only show toasts in admin dashboard, not on subscriber or showcase pages
  const path = window.location.pathname;
  return path.includes('/admin') && 
         !path.includes('/subscriber') && 
         !path.includes('/showcase');
};

// Set up event listeners with proper cleanup and reduced frequency
const setupEventListeners = () => {
  // Check if we're on a public-facing page
  const isPublicPage = window.location.pathname.includes('/showcase') || 
                       !window.location.hostname.includes('lovableproject.com');
  
  // Store timeout ID for cleanup
  let timeoutId: number | null = null;
  let lastUpdateTime = Date.now();
  const updateThrottleTime = 30 * 60 * 1000; // 30 minutes instead of 10 minutes
  
  // Add custom event dispatcher for elements update
  // Don't dispatch on initial load to avoid immediate toasts
  // Only dispatch if not on public page
  if (!isPublicPage) {
    // Use a one-time timeout instead of an immediate dispatch
    // Set the timeout to a much longer delay (10 minutes)
    const initialTimeoutId = window.setTimeout(() => {
      window.dispatchEvent(new CustomEvent('funnelDepotUpdate'));
      lastUpdateTime = Date.now();
    }, 10 * 60 * 1000); // 10 minutes instead of 10 seconds
    
    // Set up the timeout with proper window references at a much lower frequency
    const scheduleNextUpdate = () => {
      if (timeoutId !== null) {
        window.clearTimeout(timeoutId);
      }
      
      timeoutId = window.setTimeout(() => {
        // Re-check if we're on a public page (in case of navigation)
        const currentIsPublicPage = window.location.pathname.includes('/showcase') || 
                                   !window.location.hostname.includes('lovableproject.com');
        
        // Only dispatch update events in admin areas
        if (!currentIsPublicPage && !document.hidden) {
          window.dispatchEvent(new CustomEvent('funnelDepotUpdate'));
          lastUpdateTime = Date.now();
        }
        
        // Schedule next update
        scheduleNextUpdate();
      }, updateThrottleTime);
    };
    
    scheduleNextUpdate();
    
    // Add visibility change handler to update on tab focus after long absence
    const handleVisibilityChange = () => {
      if (!document.hidden && Date.now() - lastUpdateTime > updateThrottleTime) {
        window.dispatchEvent(new CustomEvent('funnelDepotUpdate'));
        lastUpdateTime = Date.now();
      }
    };
    
    document.addEventListener('visibilitychange', handleVisibilityChange);
    
    // Return cleanup function that handles all timeouts and listeners
    return () => {
      if (timeoutId !== null) {
        window.clearTimeout(timeoutId);
      }
      window.clearTimeout(initialTimeoutId);
      document.removeEventListener('visibilitychange', handleVisibilityChange);
    };
  }
  
  // Return empty cleanup function if no listeners were added
  return () => {};
};

// Render the app with DataProvider
ReactDOM.createRoot(rootElement).render(
  <React.StrictMode>
    <ErrorBoundary>
      <DataProvider>
        <App />
        <Toaster 
          position="top-right" 
          closeButton
          richColors
          duration={2000} // Reduced from 3000 to 2000 ms
          toastOptions={{
            // Default options for all toasts
            className: 'toast-item',
            style: { maxWidth: '350px' },
            // Filter toasts based on route
            // We can't use 'visibleOnly' as it's not a valid property
            // Instead, implement custom filtering in the toast hook
          }}
        />
      </DataProvider>
    </ErrorBoundary>
  </React.StrictMode>
)

// Execute post-render tasks
removeInitialLoader();
setTimeout(initThemeWithDelay, 1000);

// Set up event listeners with proper cleanup handler
const cleanupEventListeners = setupEventListeners();

// Ensure cleanup happens when the app is unloaded
window.addEventListener('beforeunload', () => {
  cleanupEventListeners();
  
  // Clean up memory utilities
  if (process.env.NODE_ENV === 'development') {
    cleanupMemoryUtils();
  }
});

// Add cleanup for when the app is unmounted (for hot reloading in development)
if (import.meta.hot) {
  import.meta.hot.dispose(() => {
    cleanupEventListeners();
    
    // Clean up memory utilities
    if (process.env.NODE_ENV === 'development') {
      cleanupMemoryUtils();
    }
  });
}
